import os
import re
import toml
import ast
from contextlib import contextmanager
from tuitka.constants import PYTHON_VERSION
from dataclasses import dataclass, field
from pathlib import Path
from typing import Optional
import shutil

import sys
import platform
from os import chdir


platform_name = platform.system().lower()


def error(message: str, title: str = "Error", subtitle: Optional[str] = None):
    from rich import print
    from rich.panel import Panel

    panel_kwargs = {"title": title, "border_style": "red"}
    if subtitle:
        panel_kwargs["subtitle"] = subtitle

    print(Panel.fit(message, **panel_kwargs))


@contextmanager
def chdir_context(path: Path):
    original_cwd = Path.cwd()
    try:
        chdir(path)
        yield
    finally:
        chdir(original_cwd)


@dataclass
class DependenciesMetadata:
    dependencies: list[str]
    requirements_path: Optional[Path] = None
    detected_imports: list[str] = field(default_factory=list)

    def to_pep_723(self) -> str:
        script_metadata = {"dependencies": self.dependencies}
        toml_content = toml.dumps(script_metadata)
        lines = ["# /// script"]
        lines.extend(f"# {line}" for line in toml_content.strip().splitlines())
        lines.append("# ///")
        return "\n".join(lines)

    @contextmanager
    def temp_pep_723_file(self, file_path: Path):
        original_content = file_path.read_text(encoding="utf-8")
        try:
            pep_723_block = self.to_pep_723()
            new_content = f"{pep_723_block}\n\n{original_content}"
            file_path.write_text(new_content, encoding="utf-8")
            yield file_path
        finally:
            file_path.write_text(original_content, encoding="utf-8")


def extract_dependencies_from_table(dep_table: dict) -> list[str]:
    deps = []
    for name, value in dep_table.items():
        if name == "python":
            continue
        if isinstance(value, str):
            deps.append(
                f"{name}{value if value.startswith('[') else '==' + value.lstrip('=<>!~ ')}"
            )
        elif isinstance(value, dict):
            version = value.get("version")
            extras = value.get("extras")
            dep_str = name
            if extras and isinstance(extras, list):
                dep_str += "[{}]".format(",".join(extras))
            if version:
                dep_str += f"=={version.lstrip('=<>!~ ')}"
            deps.append(dep_str)
        else:
            deps.append(str(name))
    return deps


class DependencyParser:
    # All credit due to https://github.com/ftnext/pep723
    # ref: https://peps.python.org/pep-0723/#specification
    PEP_723_REGEX = (
        r"(?m)^# /// (?P<type>[a-zA-Z0-9-]+)$\s(?P<content>(^#(| .*)$\s)+)^# ///$"
    )

    def __init__(self, path: Path):
        self.path = path

    def parse_pep_723(self, script: str) -> list[str]:
        name = "script"
        matches = list(
            filter(
                lambda m: m.group("type") == name,
                re.finditer(self.PEP_723_REGEX, script),
            )
        )
        if not matches:
            return []
        if len(matches) > 1:
            raise ValueError(f"Multiple {name} blocks found. You can write only one")
        group = matches[0].groupdict()
        content = "".join(
            line[2:] for line in group["content"].splitlines(keepends=True)
        )
        return toml.loads(content).get("dependencies", [])

    def parse_requirements_txt(self, requirements_path: Path) -> list[str]:
        lines = requirements_path.read_text(encoding="utf-8").splitlines()
        return [
            line.strip() for line in lines if line.strip() and not line.startswith("#")
        ]

    def parse_pyproject_toml(self, pyproject_path: Path) -> list[str]:
        content = pyproject_path.read_text(encoding="utf-8")
        pyproject_dict = toml.loads(content)
        deps = []
        tool = pyproject_dict.get("tool", {})

        # Poetry dependencies
        poetry_deps = tool.get("poetry", {}).get("dependencies", {})
        if poetry_deps:
            deps += extract_dependencies_from_table(poetry_deps)

        # Hatch dependencies
        hatch_deps = tool.get("hatch", {}).get("metadata", {}).get("dependencies", [])
        if hatch_deps:
            deps += [d for d in hatch_deps if isinstance(d, str)]

        # PEP 621 project dependencies
        project_deps = pyproject_dict.get("project", {}).get("dependencies", [])
        if project_deps:
            deps += [d for d in project_deps if isinstance(d, str)]

        return list(set(deps))

    def scan_for_imports(self, script: str) -> list[str]:
        try:
            tree = ast.parse(script)
            imports = set()
            for node in ast.walk(tree):
                if isinstance(node, ast.Import):
                    for alias in node.names:
                        imports.add(alias.name.split(".")[0])
                elif isinstance(node, ast.ImportFrom):
                    if node.module:
                        imports.add(node.module.split(".")[0])
            std_libs = set(sys.stdlib_module_names)
            return sorted(list(imports - std_libs))
        except SyntaxError:
            return []

    def parse(self) -> DependenciesMetadata:
        if not self.path or not self.path.exists():
            return DependenciesMetadata(dependencies=[])

        script = self.path.read_text(encoding="utf-8")
        detected_imports = self.scan_for_imports(script)

        dependencies = self.parse_pep_723(script)
        if dependencies:
            return DependenciesMetadata(
                dependencies=dependencies,
                requirements_path=self.path,
                detected_imports=detected_imports,
            )

        detected_imports = []
        script = self.path.read_text(encoding="utf-8")
        detected_imports = self.scan_for_imports(script)

        search_dir = self.path.parent

        # Try pyproject.toml first
        pyproject_candidate = search_dir / "pyproject.toml"
        if pyproject_candidate.exists():
            dependencies = self.parse_pyproject_toml(pyproject_candidate)
            return DependenciesMetadata(
                dependencies=dependencies,
                requirements_path=pyproject_candidate,
                detected_imports=detected_imports,
            )

        # Fallback to requirements.txt
        requirements_candidate = search_dir / "requirements.txt"
        if requirements_candidate.exists():
            dependencies = self.parse_requirements_txt(requirements_candidate)
            return DependenciesMetadata(
                dependencies=dependencies,
                requirements_path=requirements_candidate,
                detected_imports=detected_imports,
            )

        return DependenciesMetadata(dependencies=[], detected_imports=detected_imports)


def parse_dependencies(_path: str | Path) -> DependenciesMetadata:
    path = Path(_path) if isinstance(_path, str) else _path
    parser = DependencyParser(path)
    return parser.parse()


def prepare_nuitka_command(
    script_path: Path, python_version: str = PYTHON_VERSION, **nuitka_options
) -> tuple[list[str], DependenciesMetadata]:
    dependencies_metadata = parse_dependencies(script_path)
    original_is_standalone = nuitka_options.get("--standalone", False)
    original_is_onefile = nuitka_options.get("--onefile", False)
    
    if platform_name == "darwin":
        if original_is_onefile or original_is_standalone:
            nuitka_options.pop("--onefile", None)
            nuitka_options.pop("--standalone", None)
            nuitka_options["--mode"] = "app"

    is_standalone = nuitka_options.get("--standalone", False)
    is_onefile = nuitka_options.get("--onefile", False)
    is_app_mode = nuitka_options.get("--mode") == "app"

    if dependencies_metadata.detected_imports:
        auto_plugins = apply_plugins(
            dependencies_metadata.detected_imports,
            is_standalone=is_standalone,
            is_onefile=is_onefile,
            is_app_mode=is_app_mode,
        )
        for plugin_flag, enabled in auto_plugins.items():
            if plugin_flag not in nuitka_options:
                nuitka_options[plugin_flag] = enabled

    cmd = [
        "uv",
        "--python-preference",
        "system",
        "run",
        "--no-project",
        "--python",
        python_version,
        "--isolated",
    ]
    if dependencies_metadata.dependencies:
        for dependency in dependencies_metadata.dependencies:
            cmd.extend(["--with", dependency])
    cmd.extend(["--with", "nuitka", "-m", "nuitka"])

    for flag, value in nuitka_options.items():
        if value is None:
            continue
        if isinstance(value, bool) and value:
            cmd.append(flag)
        elif isinstance(value, str) and value.strip():
            cmd.append(f"{flag}={value.strip()}")
        elif isinstance(value, (list, tuple)) and value:
            for item in value:
                if isinstance(item, str) and item.strip():
                    cmd.append(f"{flag}={item.strip()}")

    cmd.append(script_path.as_posix())

    return cmd, dependencies_metadata


def create_nuitka_options_dict() -> dict[str, dict[str, dict]]:
    from collections import OrderedDict

    sys.argv.append("--help-all")
    from nuitka.OptionParsing import parser
    from nuitka.plugins.Plugins import addStandardPluginCommandLineOptions

    addStandardPluginCommandLineOptions(parser=parser, plugin_help_mode=True)
    del sys.argv[-1]

    options_dict = OrderedDict()

    if hasattr(parser, "option_list") and parser.option_list:
        general_options = OrderedDict()
        for option in parser.option_list:
            option_names = option._short_opts + option._long_opts
            option_data = {
                "names": option_names,
                "help": option.help,
                "default": getattr(option, "default", None),
                "type": getattr(option, "type", None),
                "choices": getattr(option, "choices", None),
                "dest": getattr(option, "dest", None),
                "action": getattr(option, "action", None),
                "metavar": getattr(option, "metavar", None),
            }

            primary_name = next(
                (name for name in option_names if name.startswith("--")),
                option_names[0] if option_names else str(option),
            )
            general_options[primary_name] = option_data

        if general_options:
            options_dict["General Options"] = general_options

    for group in parser.option_groups:
        group_name = group.title
        group_options = OrderedDict()

        for option in group.option_list:
            option_names = option._short_opts + option._long_opts
            option_data = {
                "names": option_names,
                "help": option.help,
                "default": getattr(option, "default", None),
                "type": getattr(option, "type", None),
                "choices": getattr(option, "choices", None),
                "dest": getattr(option, "dest", None),
                "action": getattr(option, "action", None),
                "metavar": getattr(option, "metavar", None),
            }

            primary_name = next(
                (name for name in option_names if name.startswith("--")),
                option_names[0] if option_names else str(option),
            )
            group_options[primary_name] = option_data

        options_dict[group_name] = group_options

    return options_dict


def apply_plugins(
    imports: list[str], is_standalone: bool = False, is_onefile: bool = False, is_app_mode: bool = False
) -> dict[str, bool]:
    plugins = {}

    imports_str = " ".join(imports).lower()

    qt_frameworks = {
        "pyside6": ["pyside6"],
        "pyside2": ["pyside2"],
        "pyqt6": ["pyqt6"],
        "pyqt5": ["pyqt5"],
    }

    if is_standalone or is_onefile or is_app_mode:
        for plugin_name, patterns in qt_frameworks.items():
            if any(pattern in imports_str for pattern in patterns):
                plugins[f"--enable-plugin={plugin_name}"] = True

    if any("tkinter" in imp.lower() for imp in imports):
        plugins["--enable-plugin=tk-inter"] = True

    if any("multiprocessing" in imp.lower() for imp in imports):
        plugins["--enable-plugin=multiprocessing"] = True

    return plugins


def get_default_shell() -> str:
    """Get the default shell command for the current platform."""
    if platform_name == "windows":
        return "cmd"
        if shutil.which("pwsh"):
            return "pwsh"
        elif shutil.which("powershell"):
            return "powershell"
        else:
            return "cmd"
    else:
        shell = os.environ.get("SHELL")
        if shell and shutil.which(shell):
            return shell
        for shell in ["/bin/bash", "/bin/sh", "/usr/bin/bash"]:
            if os.path.exists(shell):
                return shell
        return "sh"  # Final fallback


__all__ = [
    "prepare_nuitka_command",
    "create_nuitka_options_dict",
    "DependenciesMetadata",
    "get_default_shell",
    "apply_plugins",
]


if __name__ == "__main__":
    import json
    from rich import print

    options_dict = create_nuitka_options_dict()
    print(json.dumps(options_dict, indent=4, default=str))



# /// script
# dependencies = [
#   "toga"
# ]
# ///

import toga
import os


def button_handler(widget):
    print("hello button")


def build(app):
    box = toga.Box()
    button = toga.Button("Hello world", on_press=button_handler)
    button.style.padding = 50
    button.style.flex = 1
    box.add(button)
    return box


def main():
    app = toga.App("First App", "org.nuitka.helloworld", startup=build)

    if os.getenv("NUITKA_TEST_INTERACTIVE") != "0":
        app.main_loop()


if __name__ == "__main__":
    main()



from pathlib import Path


def get_asset_path(asset_name: str) -> Path:
    assets_dir = Path(__file__).parent
    return assets_dir / asset_name


STYLE_MAIN = get_asset_path("style.tcss")
STYLE_INLINE_APP = get_asset_path("style_inline_app.tcss")
STYLE_MODAL_FILEDIALOG = get_asset_path("style_modal_filedialog.tcss")
STYLE_MODAL_COMPILATION = get_asset_path("style_modal_compilation.tcss")
STYLE_MODAL_SETTINGS = get_asset_path("style_modal_settings.tcss")
STYLE_MODAL_SPLASHSCREEN = get_asset_path("style_modal_splashscreen.tcss")
STYLE_MODAL_SUPPORT = get_asset_path("style_modal_support.tcss")

NUITKA_LOGO = get_asset_path("logo/nuitka.png")

CONTENT_SUPPORT_NUITKA = get_asset_path("content/support_nuitka.md").read_text(
    encoding="utf-8"
)
CONTENT_COMMERCIAL = get_asset_path("content/commercial.md").read_text(encoding="utf-8")

__all__ = [
    "get_asset_path",
    "STYLE_MAIN",
    "STYLE_INLINE_APP",
    "STYLE_MODAL_FILEDIALOG",
    "STYLE_MODAL_COMPILATION",
    "STYLE_MODAL_SETTINGS",
    "STYLE_MODAL_SPLASHSCREEN",
    "STYLE_MODAL_SUPPORT",
    "NUITKA_LOGO",
    "CONTENT_SUPPORT_NUITKA",
    "CONTENT_COMMERCIAL",
]



import sys
from tuitka.tui import NuitkaTUI
from pathlib import Path
from tuitka.utils import chdir_context, error


def main() -> None:
    if len(sys.argv) > 1:
        path = Path(sys.argv[1]).resolve()
        if not path.is_file() or not path.suffix == ".py":
            error(
                f"{path} is not a valid Python file. Please provide a valid Python file or if you want to run the TUI, just run `tuitka` without arguments.",
                subtitle="Usage: tuitka <file.py>",
            )
            return

        from tuitka.inline_app import InlineCompilationApp

        default_options = {
            "--onefile": True,
            "--assume-yes-for-downloads": True,
            "--remove-output": True,
        }

        with chdir_context(path.parent):
            inline_app = InlineCompilationApp(path, **default_options)
            inline_app.run(inline=True)
            return

    app = NuitkaTUI()
    app.run()



from pathlib import Path

from textual import on
from textual.app import ComposeResult
from textual.containers import Horizontal, Vertical
from textual.reactive import reactive
from textual.screen import ModalScreen
from textual.widgets import Button, Static
from tuitka.constants import PYTHON_VERSION
from textual_tty.widgets import TextualTerminal
from tuitka.utils import prepare_nuitka_command
from tuitka.assets import STYLE_MODAL_COMPILATION
import os


class CompilationScreen(ModalScreen):
    CSS_PATH = STYLE_MODAL_COMPILATION

    compilation_finished: reactive[bool] = reactive(False, init=False)
    compilation_success: reactive[bool] = reactive(False, init=False)

    def __init__(self, python_version: str = PYTHON_VERSION, **nuitka_options) -> None:
        self.cwd = Path.cwd()
        os.chdir(self.app.script.parent)
        super().__init__()
        self.python_version = python_version
        self.nuitka_options = nuitka_options
        self.terminal = None
        self.nuitka_command = None
        self.deps_metadata = None

    def compose(self) -> ComposeResult:
        self.nuitka_command, self.deps_metadata = prepare_nuitka_command(
            self.app.script, self.python_version, **self.nuitka_options
        )

        with Vertical():
            yield TextualTerminal(
                id="compilation_terminal", command=self.nuitka_command
            )
            yield Static(
                "Compilation in progress...",
                id="status_label",
                classes="compilation-status in-progress",
            )
            with Horizontal(classes="compilation-controls"):
                yield Button("Close", variant="default", id="btn_close", disabled=True)
                yield Button("Cancel", variant="error", id="btn_cancel")

    @on(Button.Pressed)
    def handle_button_press(self, event: Button.Pressed) -> None:
        if event.button.id == "btn_close":
            self.dismiss()
        elif event.button.id == "btn_cancel":
            self.cancel_compilation()
            self.dismiss()

    def watch_compilation_finished(self, finished: bool) -> None:
        if finished:
            close_btn = self.query_one("#btn_close", Button)
            cancel_btn = self.query_one("#btn_cancel", Button)
            status_label = self.query_one("#status_label", Static)

            close_btn.disabled = False
            cancel_btn.disabled = True

            if self.compilation_success:
                status_label.update("✓ Compilation completed successfully!")
                status_label.set_class(True, "success")
                status_label.set_class(False, "in-progress")
            else:
                status_label.update("✗ Compilation failed!")
                status_label.set_class(True, "error")
                status_label.set_class(False, "in-progress")

    def on_mount(self) -> None:
        self.terminal = self.query_one("#compilation_terminal", TextualTerminal)

    def cancel_compilation(self) -> None:
        if self.terminal:
            self.terminal.stop_process()

    @on(TextualTerminal.ProcessExited)
    def on_process_exited(self, event: TextualTerminal.ProcessExited) -> None:
        self.compilation_success = event.exit_code == 0
        self.compilation_finished = True
        os.chdir(self.cwd)



from textual.app import ComposeResult
from textual.containers import Grid, Vertical
from textual.widgets import Input, RadioButton, RadioSet, Select, Static, Switch


class ModalBoolFlag(Grid):
    def __init__(
        self,
        flag: str,
        help_text: str,
        default: bool = False,
        action: str = "store_true",
        nuitka_default: bool | None = None,
    ):
        super().__init__()
        self.flag = flag
        self.help_text = help_text
        self.default = default
        self.action = action
        self.nuitka_default = nuitka_default
        self.initial_value = default

    def compose(self) -> ComposeResult:
        yield Static(self.flag)
        yield Switch(value=self.initial_value, id=f"switch_{self.flag}")

    def get_value(self):
        switch = self.query_one(f"#switch_{self.flag}", Switch)
        if self.action == "store_false":
            return not switch.value if switch.value != self.default else None
        else:
            return switch.value if switch.value != self.default else None

    def is_changed(self) -> bool:
        return self.query_one(Switch).value != self.initial_value

    def reset(self) -> None:
        self.query_one(Switch).value = self.initial_value


class ModalStringFlag(Grid):
    def __init__(self, flag: str, help_text: str, default: str = "", metavar: str = ""):
        super().__init__()
        self.flag = flag
        self.help_text = help_text
        self.default = default
        self.initial_value = default
        self.metavar = metavar

    def compose(self) -> ComposeResult:
        yield Static(self.flag)
        yield Input(
            value=str(self.initial_value) if self.initial_value else "",
            placeholder=self.metavar or self.flag.upper(),
            id=f"input_{self.flag}",
        )

    def get_value(self):
        input_widget = self.query_one(f"#input_{self.flag}", Input)
        value = input_widget.value.strip()
        return value if value and value != str(self.default) else None

    def is_changed(self) -> bool:
        return self.query_one(Input).value != self.initial_value

    def reset(self) -> None:
        self.query_one(Input).value = self.initial_value


class ModalSelectionFlag(Grid):
    def __init__(self, flag: str, help_text: str, choices: list, default=None):
        super().__init__()
        self.flag = flag
        self.help_text = help_text
        self.choices = choices
        self.default = default
        self.initial_value = default

    def compose(self) -> ComposeResult:
        yield Static(self.flag)
        options = [(choice, choice) for choice in self.choices]
        if self.initial_value:
            initial_option = self.initial_value
        else:
            initial_option = Select.BLANK
        yield Select(options, value=initial_option, id=f"select_{self.flag}")

    def get_value(self):
        select = self.query_one(f"#select_{self.flag}", Select)
        return (
            select.value
            if select.value != Select.BLANK and select.value != self.default
            else None
        )

    def is_changed(self) -> bool:
        return self.query_one(Select).value != self.initial_value

    def reset(self) -> None:
        self.query_one(Select).value = self.initial_value


class ModalRadioFlag(Grid):
    def __init__(self, flag: str, help_text: str, choices: list, default=None):
        super().__init__()
        self.flag = flag
        self.help_text = help_text
        self.choices = choices
        self.default = default
        self.initial_value = default

    def compose(self) -> ComposeResult:
        yield Static(self.flag)
        with Vertical():
            with RadioSet(id=f"radio_{self.flag}"):
                for choice in self.choices:
                    yield RadioButton(
                        choice,
                        value=(choice == self.initial_value),
                        id=f"radio_{self.flag}_{choice}",
                    )

    def get_value(self):
        radio_set = self.query_one(f"#radio_{self.flag}", RadioSet)
        if radio_set.pressed_button and radio_set.pressed_button.label:
            selected_value = str(radio_set.pressed_button.label)
            return selected_value if selected_value != self.default else None
        return None

    def is_changed(self) -> bool:
        radio_set = self.query_one(RadioSet)
        if radio_set.pressed_button:
            return str(radio_set.pressed_button.label) != self.initial_value
        return self.initial_value is not None

    def reset(self) -> None:
        radio_set = self.query_one(RadioSet)
        for button in radio_set.query(RadioButton):
            button.value = str(button.label) == self.initial_value



from random import randint, uniform, choice

from textual.app import ComposeResult
from textual.containers import Vertical
from textual.geometry import Offset
from textual.message import Message
from textual.screen import ModalScreen
from textual.widgets import Static

from tuitka.constants import SNAKE_ARTS, SPLASHSCREEN_LINKS, SNAKE_FACTS
from tuitka.assets import STYLE_MODAL_SPLASHSCREEN


class SplashScreen(ModalScreen):
    CSS_PATH = STYLE_MODAL_SPLASHSCREEN

    class Dismiss(Message):
        pass

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.snake_index = randint(0, len(SNAKE_ARTS) - 1)
        self.char_index = 0
        self.animation_timer = None

    def compose(self) -> ComposeResult:
        with Vertical(id="splash-dialog"):
            with Vertical(id="splash-content"):
                yield Static("", id="splash-art")
                yield Static("", id="splash-fact")
                yield Static(SPLASHSCREEN_LINKS, id="splash-links")
                yield Static(
                    "Press any key to skip...",
                    classes="continue-text",
                )

    def on_mount(self) -> None:
        self.set_timer(8, self.dismiss_splash)

        self.initialize_content()
        self.start_animations()

    def initialize_content(self) -> None:
        fact_widget = self.query_one("#splash-fact", Static)
        fact_widget.update(f"\n{choice(SNAKE_FACTS)}\n\n")
        self.char_index = 0

    def start_animations(self) -> None:
        art_widget = self.query_one("#splash-art", Static)
        initial_offset = self.get_random_offset(magnitude=6.0)
        art_widget.animate("offset", initial_offset, duration=5.0)
        self.set_timer(0.2, self.start_text_animation)

    def start_text_animation(self) -> None:
        if self.animation_timer:
            self.animation_timer.stop()

        self.animation_timer = self.set_interval(0.012, self.update_text)

    def update_text(self) -> None:
        target_text = SNAKE_ARTS[self.snake_index]

        if self.char_index >= len(target_text):
            if self.animation_timer:
                self.animation_timer.stop()
                self.animation_timer = None
            return

        current_text = target_text[: self.char_index + 1]
        art_widget = self.query_one("#splash-art", Static)
        art_widget.update(current_text)

        self.char_index += 1

    def get_random_offset(self, magnitude: float = 15.0) -> Offset:
        x_offset = uniform(-magnitude, magnitude)
        y_offset = uniform(-magnitude * 0.4, magnitude * 0.4)
        return Offset(x_offset, y_offset)

    def dismiss_splash(self) -> None:
        self.post_message(self.Dismiss())

    def on_key(self, event) -> None:
        self.dismiss_splash()

    def on_splash_screen_dismiss(self, _: "SplashScreen.Dismiss") -> None:
        if self.animation_timer:
            self.animation_timer.stop()
        self.dismiss()



import sys

PYTHON_VERSION = f"{sys.version_info.major}.{sys.version_info.minor}"

sss_snek = r"""
    ____                  
       / . .\                
       \  ---<     Sss...    
    \  /              
     ___/ /                
    <_____/               
"""
happy_snek_2 = r"""
        ____
       / o o\ 
      \_ v _>   Hisss...
        \   /
  ______/  /
 <_______/
"""


snek_1 = r"""
    _   
     /. .\___
    (   ---<
     \  /
   ___\/_____
  <_________/
"""


face_snek = r"""
       ____     ____     ____     ____     ____     ____    
     / . . \  / . . \  / . . \  / . . \  / . . \  / . . \    
     \  ---<  \  ---<  \  ---<  \  ---<  \  ---<  \  ---<    
      \  /      \  /      \  /      \  /      \  /      \   
______/ /______/ /______/ /______/ /______/ /______/ /_____
<_______/<_______/<_______/<_______/<_______/<_______/<_____/
"""

mythical_snek = r"""
    /^\/^\
    _|__|  O|
  \/~     \_/ \
    \_______  \ \
                `\   \_\      
                  |     |      
                  /      /       
                /     /         
              /      /          
              /     /           
            (      (        
            \      ~-_
              ~-_     ~-_
                  ~--___-~

"""

SNAKE_ARTS = [sss_snek, happy_snek_2, snek_1, face_snek, mythical_snek]
SNAKE_FACTS = [
    "Did you know? Python was created as a Christmas hobby project in December 1989!",
    "Fun fact: Python's name comes from Monty Python's Flying Circus—not the snake!",
    "Easter egg: 'import antigravity'",
    "Run `python -m __hello__` for a surprise.",
    "Try `from __future__ import braces`",
    "Enable FLUFL: `from __future__ import barry_as_FLUFL`",
    "Hashing infinity isn't boring: `hash(float('inf'))` == 314159 (π digits).",
    "Declare `__peg_parser__ = ...` in REPL and Python says: 'SyntaxError: You found it!'",
    "Python supports fancy loop syntax: 'else' runs if no `break` in loop.",
    "Odd bug (pre-Python 3.5): `time(0,0,0)` evaluated to False. Not anymore!",
    "Python's `__debug__` is True by default, but can be set to False with `-O` flag.",
    "try `import this` for the Zen of Python, a collection of guiding principles.",
    "Nuitka is short for Annuitka, which is the nickname Nuitka's author uses for his wife Anna.",
]

SPLASHSCREEN_LINKS = (
    "\n\nNuitka Github: \n\n[#fbdd58 underline]https://github.com/Nuitka/Nuitka[/]\n\n"
    "Tuitka Github: \n\n[#fbdd58 underline]https://github.com/KRRT7/tuitka[/]"
)



from textual import on
from textual.app import ComposeResult
from textual.containers import Horizontal, ScrollableContainer
from textual.screen import ModalScreen
from textual.widgets import Button, Collapsible, Input, Static

from tuitka.utils import create_nuitka_options_dict
from tuitka.assets import STYLE_MODAL_SETTINGS
from .settings_widgets import (
    ModalBoolFlag,
    ModalStringFlag,
    ModalSelectionFlag,
    ModalRadioFlag,
)


class NuitkaSettingsScreen(ModalScreen[dict | None]):
    CSS_PATH = STYLE_MODAL_SETTINGS

    def __init__(self, initial_settings: dict | None = None):
        super().__init__()
        self.current_settings = initial_settings or {}
        self.flag_widgets: list[
            ModalBoolFlag | ModalStringFlag | ModalSelectionFlag | ModalRadioFlag
        ] = []

        self.nuitka_options = create_nuitka_options_dict()

    def compose(self) -> ComposeResult:
        yield Static("Nuitka Settings", classes="settings-header")
        with Horizontal(id="filter-controls"):
            yield Input(
                placeholder="Search settings...",
                id="search_input",
                classes="filter-input",
            )
        with ScrollableContainer(id="settings-container"):
            for category, options in self.nuitka_options.items():
                with Collapsible(title=category):
                    for flag, config in options.items():
                        if self.should_skip_flag(flag, config):
                            continue

                        widget = self._create_flag_widget(flag, config)
                        if widget:
                            self.flag_widgets.append(widget)
                            yield widget

        with Horizontal(classes="settings-controls"):
            yield Button("Save", variant="success", id="save_button")
            yield Button("Cancel", variant="error", id="cancel_button")

    def should_skip_flag(self, flag: str, config: dict) -> bool:
        skip_flags = {
            "--help",
            "-h",
            "--module",
            "--version",
            "--standalone",
            "--onefile",
        }

        if flag in skip_flags:
            return True

        if config.get("action") == "help":
            return True

        if config.get("help") == "SUPPRESSHELP":
            return True

        return False

    def _create_flag_widget(self, flag: str, config: dict):
        action = config.get("action", "store")
        flag_type = config.get("type")
        choices = config.get("choices")
        default = config.get("default")
        help_text = config.get("help", "")
        metavar = config.get("metavar", "")

        current_value = self.current_settings.get(flag)

        if action in ["store_true", "store_false"]:
            widget_default = (
                current_value
                if current_value is not None
                else (default if isinstance(default, bool) else False)
            )
            widget = ModalBoolFlag(
                flag, help_text, widget_default, action=action, nuitka_default=default
            )
            widget.tooltip = help_text
            return widget

        elif choices:
            if "mode" in flag.lower() or flag in ["--mode"]:
                widget_default = current_value if current_value is not None else default
                widget = ModalRadioFlag(flag, help_text, choices, widget_default)
                widget.tooltip = help_text
                return widget
            else:
                widget_default = current_value if current_value is not None else default
                widget = ModalSelectionFlag(flag, help_text, choices, widget_default)
                widget.tooltip = help_text
                return widget

        elif action in ["store", "append"] and flag_type == "string":
            widget_default = (
                current_value
                if current_value is not None
                else (default if isinstance(default, str) else "")
            )
            widget = ModalStringFlag(flag, help_text, widget_default, metavar)
            widget.tooltip = help_text
            return widget

        return None

    @on(Input.Changed, "#search_input")
    def on_search_changed(self, event: Input.Changed) -> None:
        search_term = event.value.lower().strip()
        self.filter_settings(search_term)

    def filter_settings(self, search_term: str) -> None:
        query_selector = (
            "ModalBoolFlag, ModalStringFlag, ModalSelectionFlag, ModalRadioFlag"
        )
        all_widgets = self.query(query_selector)
        all_collapsibles = self.query(Collapsible)

        if not search_term:
            for widget in all_widgets:
                widget.display = True
            for collapsible in all_collapsibles:
                collapsible.display = True
                collapsible.collapsed = True
            self.query_one("#settings-container", ScrollableContainer).scroll_home(
                animate=False
            )
            return

        for collapsible in all_collapsibles:
            collapsible.display = False

        for widget in all_widgets:
            match = (
                search_term in widget.flag.lower()
                or search_term in widget.help_text.lower()
            )
            widget.display = match
            if match:
                parent = widget.parent
                while parent is not None and not isinstance(parent, Collapsible):
                    parent = parent.parent
                if parent is not None:
                    parent.display = True
                    parent.collapsed = False
        self.query_one("#settings-container", ScrollableContainer).scroll_home(
            animate=False
        )

    @on(Button.Pressed, "#save_button")
    def on_save_pressed(self) -> None:
        settings = {}

        for widget in self.flag_widgets:
            value = widget.get_value()
            if value is not None:
                settings[widget.flag] = value

        self.dismiss(settings)

    @on(Button.Pressed, "#cancel_button")
    def on_cancel_pressed(self) -> None:
        self.dismiss(None)



from textual.app import ComposeResult
from textual.containers import Vertical
from textual.widgets import Static
from rich_pixels import Pixels
from tuitka.assets import NUITKA_LOGO


class NuitkaHeader(Vertical):
    DEFAULT_CSS = """
    NuitkaHeader {
        height: auto;
        width: 1fr;
    }
    
    NuitkaHeader #image {
        text-align: center;
        content-align: center middle;
        margin-bottom: 1;
    }
    
    NuitkaHeader #website-link {
        text-align: center;
        margin: 1 0;
    }
    """

    def compose(self) -> ComposeResult:
        yield Static(Pixels.from_image_path(NUITKA_LOGO, (20, 20)), id="image")
        yield Static("[dim]https://nuitka.net[/dim]", id="website-link")



from pathlib import Path
from typing import Iterable

from textual import on
from textual.app import ComposeResult
from textual.containers import Horizontal, Vertical
from textual.reactive import reactive
from textual.screen import ModalScreen
from textual.widgets import Button, DirectoryTree, Input

from tuitka.assets import STYLE_MODAL_FILEDIALOG


class CustomDirectoryTree(DirectoryTree):
    def filter_paths(self, paths: Iterable[Path]) -> Iterable[Path]:
        exclude_patterns = (".", "__pycache__")
        return [path for path in paths if not path.name.startswith(exclude_patterns)]


class FileDialogScreen(ModalScreen[str | None]):
    CSS_PATH = STYLE_MODAL_FILEDIALOG

    dir_root: reactive[Path] = reactive(Path.cwd(), init=False)
    selected_py_file: reactive[str] = reactive("", init=False)

    def on_mount(self) -> None:
        self.query_one(Input).focus()

    def compose(self) -> ComposeResult:
        with Vertical():
            with Horizontal(classes="path-controls"):
                yield Input(
                    value=self.dir_root.as_posix(),
                    placeholder="Enter directory path",
                    id="path_input",
                )
                yield Button("🏠", variant="default", id="btn_home", tooltip="Home")
                yield Button(
                    "📁", variant="default", id="btn_cwd", tooltip="Current Dir"
                )

            yield CustomDirectoryTree(path=self.dir_root, id="file_tree")

            with Horizontal(id="horizontal-navigation"):
                yield Button(
                    "Select", variant="success", id="btn_select", disabled=True
                )
                yield Button("Cancel", variant="default", id="btn_cancel")

    @on(Button.Pressed)
    def handle_button_press(self, event: Button.Pressed) -> None:
        button_id = event.button.id

        if button_id == "btn_home":
            self._set_directory(Path.home())
        elif button_id == "btn_cwd":
            self._set_directory(Path.cwd())
        elif button_id == "btn_select":
            self.dismiss(result=self.selected_py_file)
        elif button_id == "btn_cancel":
            self.dismiss()

    @on(Input.Changed, "#path_input")
    def handle_path_change(self, event: Input.Changed) -> None:
        try:
            path = Path(event.value)
            if path.exists() and path.is_dir():
                self.dir_root = path
        except (OSError, ValueError):
            pass

    @on(DirectoryTree.FileSelected)
    def handle_file_selection(self, event: DirectoryTree.FileSelected) -> None:
        if event.path.suffix == ".py":
            self.selected_py_file = event.path.as_posix()
            self.query_one("#btn_select").disabled = False
        else:
            self.selected_py_file = ""
            self.query_one("#btn_select").disabled = True

    def watch_dir_root(self) -> None:
        if self.dir_root.exists() and self.dir_root.is_dir():
            self.query_one("#file_tree", CustomDirectoryTree).path = self.dir_root
            self.query_one("#path_input", Input).value = self.dir_root.as_posix()

    def _set_directory(self, path: Path) -> None:
        self.dir_root = path



from textual.app import App, ComposeResult
from textual.reactive import reactive
from textual.widgets import Footer, Header

from tuitka.assets import STYLE_MAIN
from tuitka.widgets.modals import SplashScreen, SupportNuitkaModal
from tuitka.widgets.script_input import ScriptInputWidget


class NuitkaTUI(App):
    CSS_PATH = STYLE_MAIN
    TITLE = "Tuitka - Nuitka Terminal UI"
    script: reactive[str] = reactive("", init=False)

    BINDINGS = [
        ("ctrl+s", "show_support", "Support Nuitka"),
    ]

    def on_mount(self) -> None:
        self.push_screen(SplashScreen())

    def action_show_support(self) -> None:
        self.push_screen(SupportNuitkaModal())

    def compose(self) -> ComposeResult:
        yield Header(show_clock=True)
        yield ScriptInputWidget()
        yield Footer()



from pathlib import Path
from textual.app import App, ComposeResult
from textual.containers import Vertical
from textual.reactive import reactive
from tuitka.constants import PYTHON_VERSION
from textual import on

from tuitka.assets import STYLE_INLINE_APP
from tuitka.widgets.nuitka_header import NuitkaHeader
from tuitka.utils import prepare_nuitka_command
from textual_tty.widgets import TextualTerminal


class InlineCompilationApp(App):
    CSS_PATH = STYLE_INLINE_APP

    compilation_finished: reactive[bool] = reactive(False, init=False)

    def __init__(
        self, python_file: Path, python_version: str = PYTHON_VERSION, **nuitka_options
    ):
        super().__init__()
        self.python_file = python_file
        self.python_version = python_version
        self.nuitka_options = nuitka_options
        self.terminal = None
        self.nuitka_command, self.deps_metadata = prepare_nuitka_command(
            python_file, python_version, **nuitka_options
        )

    def compose(self) -> ComposeResult:
        with Vertical(id="terminal-container"):
            yield NuitkaHeader()
            yield TextualTerminal(
                id="compilation_terminal", command=self.nuitka_command
            )

    def on_mount(self) -> None:
        self.terminal = self.query_one("#compilation_terminal", TextualTerminal)

    @on(TextualTerminal.ProcessExited)
    def on_process_exited(self, event: TextualTerminal.ProcessExited) -> None:
        self.compilation_finished = True
        if not self.compilation_finished:
            return
        self.set_timer(5.0, self.exit)



from textual.app import ComposeResult
from textual.containers import Vertical
from textual.widgets import Static, LoadingIndicator


class CompilationStatusWidget(Vertical):
    DEFAULT_CSS = """
    CompilationStatusWidget {
        height: auto;
        width: 1fr;
    }
    
    .status-container {
        height: auto;
        margin: 1 2;
        align: center middle;
    }
    
    .status-loading {
        height: 4;
    }
    
    .status-message {
        text-align: center;
        margin: 0 2;
        height: 1;
    }
    
    .status-success {
        color: $success;
    }
    
    .status-error {
        color: $error;
    }
    
    .status-in-progress {
        color: $text-muted;
    }
    """

    def __init__(self, initial_message: str = "Initializing compilation..."):
        super().__init__()
        self.initial_message = initial_message

    def compose(self) -> ComposeResult:
        with Vertical(classes="status-container status-loading"):
            yield LoadingIndicator(id="status_loading")
            yield Static(
                self.initial_message, id="status_message", classes="status-message"
            )

    def update_status(self, message: str, state: str = "in-progress") -> None:
        status = self.query_one("#status_message", Static)
        status.update(message)

        status.remove_class("status-in-progress", "status-success", "status-error")

        if state == "success":
            status.add_class("status-success")
        elif state == "error":
            status.add_class("status-error")
        else:
            status.add_class("status-in-progress")

    def hide_loading(self) -> None:
        self.query_one("#status_loading", LoadingIndicator).display = False



from .compilation import CompilationScreen
from .file_dialog import FileDialogScreen
from .settings import NuitkaSettingsScreen
from .settings_widgets import (
    ModalBoolFlag,
    ModalRadioFlag,
    ModalSelectionFlag,
    ModalStringFlag,
)
from .splash import SplashScreen
from .support import SupportNuitkaModal

__all__ = [
    "CompilationScreen",
    "FileDialogScreen",
    "ModalBoolFlag",
    "ModalRadioFlag",
    "ModalSelectionFlag",
    "ModalStringFlag",
    "NuitkaSettingsScreen",
    "SplashScreen",
    "SupportNuitkaModal",
]



from tuitka.utils import DependencyParser


def test_dependency_parser_no_deps(tmp_path):
    p = tmp_path / "script.py"
    p.write_text("import os\nimport sys\n")
    parser = DependencyParser(p)
    deps = parser.parse()
    assert deps.dependencies == []


def test_dependency_parser_with_pep723_deps(tmp_path):
    p = tmp_path / "script.py"
    p.write_text("# /// script\n# dependencies = [\"requests\", \"textual\"]\n# ///\n\nimport os\nimport sys\nimport requests\nimport textual\n")
    parser = DependencyParser(p)
    deps = parser.parse()
    assert sorted(deps.dependencies) == ["requests", "textual"]


def test_dependency_parser_with_requirements_txt(tmp_path):
    p = tmp_path / "script.py"
    p.write_text("import os\nimport sys\nimport requests\nimport textual\n")
    r = tmp_path / "requirements.txt"
    r.write_text("requests\ntextual\n")
    parser = DependencyParser(p)
    deps = parser.parse()
    assert sorted(deps.dependencies) == ["requests", "textual"]


def test_dependency_parser_with_pyproject_toml(tmp_path):
    p = tmp_path / "script.py"
    p.write_text("import os\nimport sys\nimport requests\nimport textual\n")
    t = tmp_path / "pyproject.toml"
    t.write_text("[project]\n    dependencies = [\"requests\", \"textual\"]\n")
    parser = DependencyParser(p)
    deps = parser.parse()
    assert sorted(deps.dependencies) == ["requests", "textual"]



from textual import on
from textual.app import ComposeResult
from textual.containers import Horizontal, Vertical
from textual.screen import ModalScreen
from textual.widgets import Button, Markdown, TabbedContent, TabPane

from tuitka.assets import (
    STYLE_MODAL_SUPPORT,
    CONTENT_SUPPORT_NUITKA,
    CONTENT_COMMERCIAL,
)


class SupportNuitkaModal(ModalScreen):
    CSS_PATH = STYLE_MODAL_SUPPORT

    BINDINGS = [
        ("s", "show_tab('support')", "Support Nuitka"),
        ("c", "show_tab('commercial')", "Commercial"),
        ("escape", "dismiss", "Close"),
    ]

    def compose(self) -> ComposeResult:
        with Vertical(id="support-dialog"):
            with TabbedContent(initial="support"):
                with TabPane("Support Nuitka", id="support"):
                    yield Markdown(CONTENT_SUPPORT_NUITKA)

                with TabPane("Commercial", id="commercial"):
                    yield Markdown(CONTENT_COMMERCIAL)

            with Horizontal(classes="support-controls"):
                yield Button("Close", variant="primary", id="close_button")

    @on(Button.Pressed, "#close_button")
    def on_close_pressed(self) -> None:
        self.dismiss()

    def action_show_tab(self, tab: str) -> None:
        tabbed_content = self.query_one(TabbedContent)
        tabbed_content.active = tab

    def action_dismiss(self) -> None:
        self.dismiss()




# Copyright (C) 2013 Riverbank Computing Limited.
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause

"""PySide6 port of the opengl/hellogl2 example from Qt v5.x"""
from argparse import ArgumentParser, RawTextHelpFormatter
import ctypes
import math
import sys
from PySide6.QtCore import Signal, Slot, Qt, QSize, QPointF
from PySide6.QtGui import (QVector3D, QOpenGLFunctions,
    QMatrix4x4, QOpenGLContext, QSurfaceFormat, QVector3DList)
from PySide6.QtOpenGL import (QOpenGLVertexArrayObject, QOpenGLBuffer,
    QOpenGLShaderProgram, QOpenGLShader)
from PySide6.QtWidgets import (QApplication, QWidget, QMessageBox, QHBoxLayout,
    QSlider)
from PySide6.QtOpenGLWidgets import QOpenGLWidget

from shiboken6 import VoidPtr

try:
    from OpenGL import GL
except ImportError:
    app = QApplication(sys.argv)
    message_box = QMessageBox(QMessageBox.Critical, "OpenGL hellogl",
                                         "PyOpenGL must be installed to run this example.",
                                         QMessageBox.Close)
    message_box.setDetailedText("Run:\npip install PyOpenGL PyOpenGL_accelerate")
    message_box.exec()
    sys.exit(1)


class Window(QWidget):
    def __init__(self, transparent, parent=None):
        QWidget.__init__(self, parent)

        if transparent:
            self.setAttribute(Qt.WA_TranslucentBackground)
            self.setAttribute(Qt.WA_NoSystemBackground, False)

        self._gl_widget = GLWidget(transparent)

        self._x_slider = self.create_slider()
        self._x_slider.valueChanged.connect(self._gl_widget.set_xrotation)
        self._gl_widget.x_rotation_changed.connect(self._x_slider.setValue)

        self._y_slider = self.create_slider()
        self._y_slider.valueChanged.connect(self._gl_widget.set_yrotation)
        self._gl_widget.y_rotation_changed.connect(self._y_slider.setValue)

        self._z_slider = self.create_slider()
        self._z_slider.valueChanged.connect(self._gl_widget.set_zrotation)
        self._gl_widget.z_rotation_changed.connect(self._z_slider.setValue)

        main_layout = QHBoxLayout()
        main_layout.addWidget(self._gl_widget)
        main_layout.addWidget(self._x_slider)
        main_layout.addWidget(self._y_slider)
        main_layout.addWidget(self._z_slider)
        self.setLayout(main_layout)

        self._x_slider.setValue(15 * 16)
        self._y_slider.setValue(345 * 16)
        self._z_slider.setValue(0 * 16)

        self.setWindowTitle(self.tr("Hello GL"))

    def create_slider(self):
        slider = QSlider(Qt.Vertical)

        slider.setRange(0, 360 * 16)
        slider.setSingleStep(16)
        slider.setPageStep(15 * 16)
        slider.setTickInterval(15 * 16)
        slider.setTickPosition(QSlider.TicksRight)
        return slider

    def keyPressEvent(self, event):
        if event.key() == Qt.Key_Escape:
            self.close()
        else:
            super(Window, self).keyPressEvent(event)


class Logo():
    def __init__(self):
        self.m_data = QVector3DList()
        self.m_data.reserve(5000)

        x1 = +0.06
        y1 = -0.14
        x2 = +0.14
        y2 = -0.06
        x3 = +0.08
        y3 = +0.00
        x4 = +0.30
        y4 = +0.22

        self.quad(x1, y1, x2, y2, y2, x2, y1, x1)
        self.quad(x3, y3, x4, y4, y4, x4, y3, x3)

        self.extrude(x1, y1, x2, y2)
        self.extrude(x2, y2, y2, x2)
        self.extrude(y2, x2, y1, x1)
        self.extrude(y1, x1, x1, y1)
        self.extrude(x3, y3, x4, y4)
        self.extrude(x4, y4, y4, x4)
        self.extrude(y4, x4, y3, x3)

        NUM_SECTORS = 100

        for i in range(NUM_SECTORS):
            angle = (i * 2 * math.pi) / NUM_SECTORS
            x5 = 0.30 * math.sin(angle)
            y5 = 0.30 * math.cos(angle)
            x6 = 0.20 * math.sin(angle)
            y6 = 0.20 * math.cos(angle)

            angle = ((i + 1) * 2 * math.pi) / NUM_SECTORS
            x7 = 0.20 * math.sin(angle)
            y7 = 0.20 * math.cos(angle)
            x8 = 0.30 * math.sin(angle)
            y8 = 0.30 * math.cos(angle)

            self.quad(x5, y5, x6, y6, x7, y7, x8, y8)

            self.extrude(x6, y6, x7, y7)
            self.extrude(x8, y8, x5, y5)

    def const_data(self):
        return self.m_data.constData()

    def count(self):
        return len(self.m_data) * 3

    def vertex_count(self):
        return self.count() / 6

    def quad(self, x1, y1, x2, y2, x3, y3, x4, y4):
        n = QVector3D.normal(QVector3D(x4 - x1, y4 - y1, 0), QVector3D(x2 - x1, y2 - y1, 0))

        self.add(QVector3D(x1, y1, -0.05), n)
        self.add(QVector3D(x4, y4, -0.05), n)
        self.add(QVector3D(x2, y2, -0.05), n)

        self.add(QVector3D(x3, y3, -0.05), n)
        self.add(QVector3D(x2, y2, -0.05), n)
        self.add(QVector3D(x4, y4, -0.05), n)

        n = QVector3D.normal(QVector3D(x1 - x4, y1 - y4, 0), QVector3D(x2 - x4, y2 - y4, 0))

        self.add(QVector3D(x4, y4, 0.05), n)
        self.add(QVector3D(x1, y1, 0.05), n)
        self.add(QVector3D(x2, y2, 0.05), n)

        self.add(QVector3D(x2, y2, 0.05), n)
        self.add(QVector3D(x3, y3, 0.05), n)
        self.add(QVector3D(x4, y4, 0.05), n)

    def extrude(self, x1, y1, x2, y2):
        n = QVector3D.normal(QVector3D(0, 0, -0.1), QVector3D(x2 - x1, y2 - y1, 0))

        self.add(QVector3D(x1, y1, 0.05), n)
        self.add(QVector3D(x1, y1, -0.05), n)
        self.add(QVector3D(x2, y2, 0.05), n)

        self.add(QVector3D(x2, y2, -0.05), n)
        self.add(QVector3D(x2, y2, 0.05), n)
        self.add(QVector3D(x1, y1, -0.05), n)

    def add(self, v, n):
        self.m_data.append(v)
        self.m_data.append(n)


class GLWidget(QOpenGLWidget, QOpenGLFunctions):
    x_rotation_changed = Signal(int)
    y_rotation_changed = Signal(int)
    z_rotation_changed = Signal(int)

    def __init__(self, transparent, parent=None):
        QOpenGLWidget.__init__(self, parent)
        QOpenGLFunctions.__init__(self)

        self._transparent = transparent
        self._core = QSurfaceFormat.defaultFormat().profile() == QSurfaceFormat.CoreProfile

        self._x_rot = 0
        self._y_rot = 0
        self._z_rot = 0
        self._last_pos = QPointF()
        self.logo = Logo()
        self.vao = QOpenGLVertexArrayObject()
        self._logo_vbo = QOpenGLBuffer()
        self.program = QOpenGLShaderProgram()
        self._proj_matrix_loc = 0
        self._mv_matrix_loc = 0
        self._normal_matrix_loc = 0
        self._light_pos_loc = 0
        self.proj = QMatrix4x4()
        self.camera = QMatrix4x4()
        self.world = QMatrix4x4()
        if transparent:
            fmt = self.format()
            fmt.setAlphaBufferSize(8)
            self.setFormat(fmt)

    def x_rotation(self):
        return self._x_rot

    def y_rotation(self):
        return self._y_rot

    def z_rotation(self):
        return self._z_rot

    def minimumSizeHint(self):
        return QSize(50, 50)

    def sizeHint(self):
        return QSize(400, 400)

    def normalize_angle(self, angle):
        while angle < 0:
            angle += 360 * 16
        while angle > 360 * 16:
            angle -= 360 * 16
        return angle

    @Slot(int)
    def set_xrotation(self, angle):
        angle = self.normalize_angle(angle)
        if angle != self._x_rot:
            self._x_rot = angle
            self.x_rotation_changed.emit(angle)
            self.update()

    @Slot(int)
    def set_yrotation(self, angle):
        angle = self.normalize_angle(angle)
        if angle != self._y_rot:
            self._y_rot = angle
            self.y_rotation_changed.emit(angle)
            self.update()

    @Slot(int)
    def set_zrotation(self, angle):
        angle = self.normalize_angle(angle)
        if angle != self._z_rot:
            self._z_rot = angle
            self.z_rotation_changed.emit(angle)
            self.update()

    @Slot()
    def cleanup(self):
        self.makeCurrent()
        self._logo_vbo.destroy()
        del self.program
        self.program = None
        self.doneCurrent()

    def vertex_shader_source_core(self):
        return """#version 150
                in vec4 vertex;
                in vec3 normal;
                out vec3 vert;
                out vec3 vertNormal;
                uniform mat4 projMatrix;
                uniform mat4 mvMatrix;
                uniform mat3 normalMatrix;
                void main() {
                   vert = vertex.xyz;
                   vertNormal = normalMatrix * normal;
                   gl_Position = projMatrix * mvMatrix * vertex;
                }"""

    def fragment_shader_source_core(self):
        return """#version 150
                in highp vec3 vert;
                in highp vec3 vertNormal;
                out highp vec4 fragColor;
                uniform highp vec3 lightPos;
                void main() {
                   highp vec3 L = normalize(lightPos - vert);
                   highp float NL = max(dot(normalize(vertNormal), L), 0.0);
                   highp vec3 color = vec3(0.39, 1.0, 0.0);
                   highp vec3 col = clamp(color * 0.2 + color * 0.8 * NL, 0.0, 1.0);
                   fragColor = vec4(col, 1.0);
                }"""

    def vertex_shader_source(self):
        return """attribute vec4 vertex;
                attribute vec3 normal;
                varying vec3 vert;
                varying vec3 vertNormal;
                uniform mat4 projMatrix;
                uniform mat4 mvMatrix;
                uniform mat3 normalMatrix;
                void main() {
                   vert = vertex.xyz;
                   vertNormal = normalMatrix * normal;
                   gl_Position = projMatrix * mvMatrix * vertex;
                }"""

    def fragment_shader_source(self):
        return """varying highp vec3 vert;
                varying highp vec3 vertNormal;
                uniform highp vec3 lightPos;
                void main() {
                   highp vec3 L = normalize(lightPos - vert);
                   highp float NL = max(dot(normalize(vertNormal), L), 0.0);
                   highp vec3 color = vec3(0.39, 1.0, 0.0);
                   highp vec3 col = clamp(color * 0.2 + color * 0.8 * NL, 0.0, 1.0);
                   gl_FragColor = vec4(col, 1.0);
                }"""

    def initializeGL(self):
        self.context().aboutToBeDestroyed.connect(self.cleanup)
        self.initializeOpenGLFunctions()
        self.glClearColor(0, 0, 0, 0 if self._transparent else 1)

        self.program = QOpenGLShaderProgram()

        if self._core:
            self._vertex_shader = self.vertex_shader_source_core()
            self._fragment_shader = self.fragment_shader_source_core()
        else:
            self._vertex_shader = self.vertex_shader_source()
            self._fragment_shader = self.fragment_shader_source()

        self.program.addShaderFromSourceCode(QOpenGLShader.Vertex, self._vertex_shader)
        self.program.addShaderFromSourceCode(QOpenGLShader.Fragment, self._fragment_shader)
        self.program.bindAttributeLocation("vertex", 0)
        self.program.bindAttributeLocation("normal", 1)
        self.program.link()

        self.program.bind()
        self._proj_matrix_loc = self.program.uniformLocation("projMatrix")
        self._mv_matrix_loc = self.program.uniformLocation("mvMatrix")
        self._normal_matrix_loc = self.program.uniformLocation("normalMatrix")
        self._light_pos_loc = self.program.uniformLocation("lightPos")

        self.vao.create()
        with QOpenGLVertexArrayObject.Binder(self.vao):
            self._logo_vbo.create()
            self._logo_vbo.bind()
            float_size = ctypes.sizeof(ctypes.c_float)
            self._logo_vbo.allocate(self.logo.const_data(), self.logo.count() * float_size)

            self.setup_vertex_attribs()

            self.camera.setToIdentity()
            self.camera.translate(0, 0, -1)

            self.program.setUniformValue(self._light_pos_loc, QVector3D(0, 0, 70))
            self.program.release()

    def setup_vertex_attribs(self):
        self._logo_vbo.bind()
        f = QOpenGLContext.currentContext().functions()
        f.glEnableVertexAttribArray(0)
        f.glEnableVertexAttribArray(1)
        float_size = ctypes.sizeof(ctypes.c_float)

        null = VoidPtr(0)
        pointer = VoidPtr(3 * float_size)
        f.glVertexAttribPointer(0, 3, int(GL.GL_FLOAT), int(GL.GL_FALSE), 6 * float_size, null)
        f.glVertexAttribPointer(1, 3, int(GL.GL_FLOAT), int(GL.GL_FALSE), 6 * float_size, pointer)
        self._logo_vbo.release()

    def paintGL(self):
        self.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT)
        self.glEnable(GL.GL_DEPTH_TEST)
        self.glEnable(GL.GL_CULL_FACE)

        self.world.setToIdentity()
        self.world.rotate(180 - (self._x_rot / 16), 1, 0, 0)
        self.world.rotate(self._y_rot / 16, 0, 1, 0)
        self.world.rotate(self._z_rot / 16, 0, 0, 1)

        with QOpenGLVertexArrayObject.Binder(self.vao):
            self.program.bind()
            self.program.setUniformValue(self._proj_matrix_loc, self.proj)
            self.program.setUniformValue(self._mv_matrix_loc, self.camera * self.world)
            normal_matrix = self.world.normalMatrix()
            self.program.setUniformValue(self._normal_matrix_loc, normal_matrix)

            self.glDrawArrays(GL.GL_TRIANGLES, 0, self.logo.vertex_count())
            self.program.release()

    def resizeGL(self, width, height):
        self.proj.setToIdentity()
        self.proj.perspective(45, width / height, 0.01, 100)

    def mousePressEvent(self, event):
        self._last_pos = event.position()

    def mouseMoveEvent(self, event):
        pos = event.position()
        dx = pos.x() - self._last_pos.x()
        dy = pos.y() - self._last_pos.y()

        if event.buttons() & Qt.LeftButton:
            self.set_xrotation(self._x_rot + 8 * dy)
            self.set_yrotation(self._y_rot + 8 * dx)
        elif event.buttons() & Qt.RightButton:
            self.set_xrotation(self._x_rot + 8 * dy)
            self.set_zrotation(self._z_rot + 8 * dx)

        self._last_pos = pos


if __name__ == '__main__':
    app = QApplication(sys.argv)
    parser = ArgumentParser(description="hellogl2", formatter_class=RawTextHelpFormatter)
    parser.add_argument('--multisample', '-m', action='store_true',
                        help='Use Multisampling')
    parser.add_argument('--coreprofile', '-c', action='store_true',
                        help='Use Core Profile')
    parser.add_argument('--transparent', '-t', action='store_true',
                        help='Transparent Windows')
    options = parser.parse_args()

    fmt = QSurfaceFormat()
    fmt.setDepthBufferSize(24)
    if options.multisample:
        fmt.setSamples(4)
    if options.coreprofile:
        fmt.setVersion(3, 2)
        fmt.setProfile(QSurfaceFormat.CoreProfile)
    QSurfaceFormat.setDefaultFormat(fmt)

    main_window = Window(options.transparent)
    main_window.resize(main_window.sizeHint())
    main_window.show()

    res = app.exec()
    sys.exit(res)


from textual import on
from textual.app import ComposeResult
from textual.containers import Vertical, Center, Container
from textual.widgets import Button, Input, Static
from textual.widgets import RadioButton, RadioSet
from tuitka.constants import PYTHON_VERSION
from tuitka.widgets.nuitka_header import NuitkaHeader
from pathlib import Path

from tuitka.widgets.modals import (
    CompilationScreen,
    FileDialogScreen,
    NuitkaSettingsScreen,
)


class ScriptInput(Input):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(
            placeholder="Enter path to Python script to compile", *args, **kwargs
        )

    def on_mount(self) -> None:
        self.focus()

    def on_input_changed(self, event: Input.Changed) -> None:
        """Update app script when input changes."""
        self.app.script = Path(self.value.strip())


class ScriptInputWidget(Container):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.custom_settings = None

    DEFAULT_CSS = """
    ScriptInputWidget {
        width: 1fr;
        height: 1fr;
        align: center middle;
        padding: 1;
    }

    #main_container {
        width: 85%;
        max-width: 100;
        height: auto;
        align: center middle;
    }

    #title_label {
        text-align: center;
        color: $text-muted;
        margin-bottom: 1;
        width: 1fr;
    }

    #input_section {
        width: 1fr;
        height: auto;
        margin-bottom: 1;
    }

    #script_input {
        width: 1fr;
        margin-bottom: 1;
    }

    #browse_button {
        width: auto;
    }

    #compilation_options_container {
        border: round $panel-lighten-2;
        width: 1fr;
        height: auto;
        padding: 1;
        margin-bottom: 1;
    }

    .group_title {
        text-align: center;
        text-style: bold;
        margin-bottom: 1;
        width: 1fr;
    }

    .sub_title {
        margin-top: 0;
        margin-bottom: 0;
        color: $text-muted;
        width: 1fr;
    }

    /* disabled for now */
    #python_version_select {
        display: none;
        width: 1fr;
        margin-top: 0;
    }

    #settings_radioset {
        height: auto;
        width: 1fr;
        margin-top: 0;
        margin-bottom: 0;
        align: center middle;
        layout: horizontal;
    }

    RadioButton {
        width: auto;
        margin: 0 1;
        background: transparent;
        border: none;
        outline: none;
        text-style: none;
    }

    RadioButton:focus {
        background: transparent;
        border: none;
        outline: none;
    }

    #compile_button_container {
        width: 1fr;
        height: auto;
        align: center middle;
        margin-top: 0;
        margin-bottom: 0;
    }

    #compile_button {
        width: auto;
    }
    """

    def compose(self) -> ComposeResult:
        with Vertical(id="main_container"):
            yield NuitkaHeader()
            yield Static(
                "Select a Python script to compile with Nuitka", id="title_label"
            )

            with Vertical(id="input_section"):
                yield ScriptInput(id="script_input")
                with Center():
                    yield Button("Browse Files", variant="primary", id="browse_button")

            with Vertical(id="compilation_options_container"):
                yield Static("Compilation Options", classes="group_title")

                yield Static("Presets", classes="sub_title")
                with RadioSet(id="settings_radioset"):
                    yield RadioButton("Onefile", id="onefile_preset", value=True)
                    yield RadioButton("Standalone", id="standalone_preset")
                    yield RadioButton("Custom", id="custom_settings")

                # Python version selection temporarily disabled - using current Python version
                # yield Static("Python Version", classes="sub_title")
                # yield Select(
                #     [
                #         ("3.8", "3.8"),
                #         ("3.9", "3.9"),
                #         ("3.10", "3.10"),
                #         ("3.11", "3.11"),
                #         ("3.12", "3.12"),
                #     ],
                #     value=PYTHON_VERSION,
                #     allow_blank=False,
                #     id="python_version_select",
                # )

            with Center(id="compile_button_container"):
                yield Button("Compile", variant="success", id="compile_button")

    def on_mount(self) -> None:
        script_input = self.query_one("#script_input", ScriptInput)
        if not script_input.value.strip():
            self.query_one("#compile_button", Button).display = False
            self.query_one("#compilation_options_container").display = False

    @on(Input.Changed, "#script_input")
    def on_script_input_changed(self, event: Input.Changed) -> None:
        # evaluate if options should show up
        should_be_visible = bool(event.value.strip())

        self.query_one("#compile_button", Button).display = should_be_visible
        self.query_one("#compilation_options_container").display = should_be_visible

    @on(Input.Submitted, "#script_input")
    def on_script_input_submitted(self, event: Input.Submitted) -> None:
        settings_radioset = self.query_one("#settings_radioset")
        if event.value.strip():
            settings_radioset.display = True

    @on(RadioSet.Changed, "#settings_radioset")
    def on_radio_changed(self, event: RadioSet.Changed) -> None:
        selected_button = event.radio_set.pressed_button
        if selected_button and selected_button.id == "custom_settings":
            self.app.push_screen(
                NuitkaSettingsScreen(self.custom_settings), self._handle_custom_settings
            )
        else:
            self.query_one("#compile_button").display = True

    @on(Button.Pressed, "#browse_button")
    def open_file_dialog(self) -> None:
        self.app.push_screen(FileDialogScreen(), callback=self._handle_file_selection)

    @on(Button.Pressed, "#compile_button")
    def start_compilation(self) -> None:
        script_input = self.query_one("#script_input", ScriptInput)
        if script_input.value.strip():
            radioset = self.query_one("#settings_radioset", RadioSet)
            selected_preset = radioset.pressed_button

            if selected_preset is None:
                return

            python_version = PYTHON_VERSION

            nuitka_options = {
                "--assume-yes-for-downloads": True,
                "--remove-output": True,
            }
            if selected_preset.id == "onefile_preset":
                nuitka_options["--onefile"] = True
            elif selected_preset.id == "standalone_preset":
                nuitka_options["--standalone"] = True
            elif selected_preset.id == "custom_settings" and self.custom_settings:
                nuitka_options = self.custom_settings

            self.app.push_screen(CompilationScreen(python_version, **nuitka_options))

    def _handle_file_selection(self, selected_file: str | None) -> None:
        if selected_file:
            self.query_one("#script_input", ScriptInput).value = selected_file
            self.app.script = selected_file
            self.query_one("#compilation_options_container").display = True

    def _handle_custom_settings(self, settings: dict | None) -> None:
        if settings:
            self.custom_settings = settings
            self.query_one("#compile_button").display = True
